from reportlab.graphics.charts.linecharts import HorizontalLineChart
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin
from reportlab.graphics.charts.barcharts import VerticalBarChart
class TableBarChart(_DrawingEditorMixin,Drawing):
    def __init__(self,width=400,height=200,*args,**kw):
        Drawing.__init__(self,width,height,*args,**kw)
        self.width = 136
        self.height = 140
        self._add(self,VerticalBarChart(),name='chart',validate=None,desc=None)
        self.chart.y = 20
        self.chart.width = self.width - 21
        self.chart.height = self.height - 24
        self.chart.categoryAxis.categoryNames = ['Spring','Summer','Autumn','Winter']
        self.chart.categoryAxis.labels.fontSize = 7
def main():
    __doc__='Test script for reportlab.tables'
    import os, inspect
    from reportlab.lib.units import inch, cm
    from reportlab.platypus.flowables import Image, PageBreak, Spacer, XBox
    from reportlab.platypus.tables import Table, TableStyle
    from reportlab.platypus.paragraph import Paragraph
    from reportlab.lib import colors
    from reportlab.platypus.xpreformatted import XPreformatted
    from reportlab.platypus.flowables import Preformatted
    from reportlab.platypus.doctemplate import SimpleDocTemplate
    from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
    from reportlab.platypus.tables import GRID_STYLE, BOX_STYLE, LABELED_GRID_STYLE, COLORED_GRID_STYLE, LIST_STYLE, LongTable
    import copy

    #monkey patch the Table.draw
    old_draw = Table.draw
    def draw(self):
        print(f'Start drawing table {self._monkeyName}')
        old_draw(self)
    Table.draw = draw
    styleSheet = getSampleStyleSheet()
    lst = []
    adding = False
    def lst_add(x):
        if adding:
            lst.append(x)
    adding = True
    data=  [['00', '01', '02', '03', '04'],
            ['10', '11', '12', '13', '14'],
            ['20', '21', '22', '23', '24'],
            ['30', '31', '32', '33', '34']]
    t=Table(data,style=[
                    ('GRID',(0,0),(-1,-1),0.5,colors.grey),
                    ('GRID',(1,1),(-2,-2),1,colors.green),
                    ('BOX',(0,0),(1,-1),2,colors.red),
                    ('BOX',(0,0),(-1,-1),2,colors.black),
                    ('LINEABOVE',(1,2),(-2,2),1,colors.blue),
                    ('LINEBEFORE',(2,1),(2,-2),1,colors.pink),
                    ('BACKGROUND', (0, 0), (0, 1), colors.pink),
                    ('BACKGROUND', (1, 1), (1, 2), colors.lavender),
                    ('BACKGROUND', (2, 2), (2, 3), colors.orange),
                    ('TEXTCOLOR',(0,-1),(-2,-1),colors.green),
                    ])
    old_line_cmds = t._linecmds
    t._monkeyName = 't'
    lst_add(Paragraph("Illustrating splits: nosplit", styleSheet['BodyText']))
    lst_add(t)
    lst_add(Spacer(0,6))
    lst_add(Paragraph("Illustrating splits: split(4in,30)", styleSheet['BodyText']))
    deepcopied = os.environ.get('DEEPCOPY','0')=='1'
    print('before split id=0X%8.8X\n%s' % (id(t._linecmds), '\n'.join([repr(_) for _ in t._linecmds])))
    tc = copy.deepcopy(t) if deepcopied else t
    if not deepcopied:
        class TableCheckSetattr(Table):
            def __setattr__(self,a,v):
                if a=='_linecmds':
                    s = '0X%8X' % id(self._linecmds) if hasattr(self,'_linecmds') else (10*'?')
                    msg = '!!!!! _linecmds %s --> 0X%8.8X' % (s,id(v))
                    print(msg)
                    #import rgb_debug
                    #rgb_stack(msg)
                super().__setattr__(a,v)
        t.__class__ = TableCheckSetattr

    for i,s in enumerate(tc.split(4*inch,30)):
        s._monkeyName = f's[{i}]'
        lst_add(s)
        lst_add(Spacer(0,6))
    if not deepcopied:
        t.__class__ = Table
    print('after  split id=0X%8.8X\n%s' % (id(t._linecmds), '\n'.join([repr(_) for _ in t._linecmds])))
    patched = '-patched' if 'splitInRow' in inspect.signature(Table.__init__).parameters else ''
    docName = 'ttablelines%s%s.pdf' % (patched,deepcopied and '-deepcopied' or '')
    def monkeyCanvas(*args,**kwds):
        '''we patch the Canvas line method so it prints info'''
        from reportlab.pdfgen.canvas import Canvas
        _old_line = Canvas.line
        Canvas._monkeyFirst = True
        def line(self, x1,y1, x2,y2):
            if Canvas._monkeyFirst:
                Canvas._monkeyFirst = False
                s = 'splitInRow' if patched else 'standard'
                if deepcopied: s = 'deepcopied '+s
                print(f'this output comes from a {s} reportlab')
            print(f'line ends={(x1,y1,x2,y2)} lineWidth={self._lineWidth} strokeColor={self._strokeColorObj.rgb()}')
            _old_line(self,x1,y1,x2,y2)
        Canvas.line = line
        return Canvas(*args,**kwds)
    SimpleDocTemplate(docName, showBoundary=1).build(lst, canvasmaker=monkeyCanvas)
    print(f'saved {docName}')

if __name__=='__main__':
    main()
